"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const board_1 = require("../board");
test('en passant', () => {
    const state = (0, __1.get_initial_state)("黒");
    // destroy the pawn at ５七 to avoid 二ポ
    (0, board_1.put_entity_at_coord_and_also_adjust_flags)(state.board, ["５", "七"], null);
    expect((0, __1.from_custom_state)([
        { piece_phase: { side: "黒", to: ["６", "五"], prof: "ポ" } },
        { piece_phase: { side: "白", to: ["５", "五"], prof: "ポ" } },
        { piece_phase: { side: "黒", to: ["５", "四"], prof: "ポ" } }
    ], state)).toEqual({
        phase: "resolved",
        hand_of_black: [],
        hand_of_white: [],
        who_goes_next: "白",
        board: [
            [
                { type: "しょ", side: "白", prof: "香", can_kumal: true },
                { type: "しょ", side: "白", prof: "桂", can_kumal: false },
                { type: "しょ", side: "白", prof: "銀", can_kumal: false },
                { type: "しょ", side: "白", prof: "金", can_kumal: false },
                { type: "王", side: "白", prof: "キ", never_moved: true, has_moved_only_once: false },
                { type: "しょ", side: "白", prof: "金", can_kumal: false },
                { type: "しょ", side: "白", prof: "銀", can_kumal: false },
                { type: "しょ", side: "白", prof: "桂", can_kumal: false },
                { type: "しょ", side: "白", prof: "香", can_kumal: true },
            ],
            [
                { type: "ス", side: "白", prof: "ル", never_moved: true },
                { type: "ス", side: "白", prof: "ナ", never_moved: true },
                { type: "ス", side: "白", prof: "ビ", never_moved: true },
                null,
                { type: "ス", side: "白", prof: "ク", never_moved: true },
                null,
                { type: "ス", side: "白", prof: "ビ", never_moved: true },
                { type: "ス", side: "白", prof: "ナ", never_moved: true },
                { type: "ス", side: "白", prof: "ル", never_moved: true },
            ],
            [
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
                null,
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
                { type: "ス", side: "白", prof: "ポ", never_moved: true },
            ],
            [null, null, null, null, { type: "ス", side: "黒", prof: "ポ", never_moved: false }, null, null, null, null,],
            [null, null, null, null, null, null, null, null, null,],
            [null, null, null, null, null, null, null, null, null,],
            [
                { type: "ス", side: "黒", prof: "ポ", never_moved: true },
                { type: "ス", side: "黒", prof: "ポ", never_moved: true },
                { type: "ス", side: "黒", prof: "ポ", never_moved: true },
                null,
                null,
                { type: "ス", side: "黒", prof: "ポ", never_moved: true },
                { type: "ス", side: "黒", prof: "ポ", never_moved: true },
                { type: "ス", side: "黒", prof: "ポ", never_moved: true },
                { type: "ス", side: "黒", prof: "ポ", never_moved: true },
            ],
            [
                { type: "ス", side: "黒", prof: "ル", never_moved: true },
                { type: "ス", side: "黒", prof: "ナ", never_moved: true },
                { type: "ス", side: "黒", prof: "ビ", never_moved: true },
                null,
                { type: "ス", side: "黒", prof: "ク", never_moved: true },
                null,
                { type: "ス", side: "黒", prof: "ビ", never_moved: true },
                { type: "ス", side: "黒", prof: "ナ", never_moved: true },
                { type: "ス", side: "黒", prof: "ル", never_moved: true },
            ],
            [
                { type: "しょ", side: "黒", prof: "香", can_kumal: true },
                { type: "しょ", side: "黒", prof: "桂", can_kumal: false },
                { type: "しょ", side: "黒", prof: "銀", can_kumal: false },
                { type: "しょ", side: "黒", prof: "金", can_kumal: false },
                { type: "王", side: "黒", prof: "キ", never_moved: true, has_moved_only_once: false },
                { type: "しょ", side: "黒", prof: "金", can_kumal: false },
                { type: "しょ", side: "黒", prof: "銀", can_kumal: false },
                { type: "しょ", side: "黒", prof: "桂", can_kumal: false },
                { type: "しょ", side: "黒", prof: "香", can_kumal: true },
            ],
        ]
    });
});
test('en passant fails', () => {
    const state = (0, __1.get_initial_state)("黒");
    // destroy the pawn at ５七 to avoid 二ポ
    (0, board_1.put_entity_at_coord_and_also_adjust_flags)(state.board, ["５", "七"], null);
    // Cannot do en passant, since there are intervening moves
    expect(() => (0, __1.from_custom_state)([
        { piece_phase: { side: "黒", to: ["６", "五"], prof: "ポ" } },
        { piece_phase: { side: "白", to: ["５", "五"], prof: "ポ" } },
        { piece_phase: { side: "黒", to: ["７", "六"], prof: "ナ" } },
        { piece_phase: { side: "白", to: ["７", "四"], prof: "ナ" } },
        { piece_phase: { side: "黒", to: ["５", "四"], prof: "ポ" } },
    ], state)).toThrowError("黒が５四ポとのことですが、そのような移動ができる黒のポーン兵は盤上にありません");
});
