import { Coordinate } from "./coordinate";
import { Side } from "./side";
export declare type Tuple9<T> = [T, T, T, T, T, T, T, T, T];
export declare type Hand = UnpromotedShogiProfession[];
export declare type Phase = "piece_phase_played" | "stone_phase_played" | "resolved";
export declare type GameState = ResolvedGameState | PiecePhasePlayed | StonePhasePlayed;
export declare type Situation = {
    board: Board;
    hand_of_black: Hand;
    hand_of_white: Hand;
};
export declare type GameEnd = {
    phase: "game_end";
    victor: Side;
    reason: "doubled_pawns" | "king_capture" | "king_suicide";
    final_situation: Situation;
} | {
    phase: "game_end";
    victor: "KarateJankenBoxing";
    reason: "both_king_dead" | "king_capture_and_doubled_pawns";
    final_situation: Situation;
};
export declare type ResolvedGameState = {
    phase: "resolved";
    board: Board;
    hand_of_black: Hand;
    hand_of_white: Hand;
    who_goes_next: Side;
};
export declare type PiecePhasePlayed = {
    phase: "piece_phase_played";
    board: Board;
    hand_of_black: Hand;
    hand_of_white: Hand;
    by_whom: Side;
};
export declare type StonePhasePlayed = {
    phase: "stone_phase_played";
    board: Board;
    hand_of_black: Hand;
    hand_of_white: Hand;
    by_whom: Side;
};
export declare type Board = Tuple9<Row>;
export declare type Row = Tuple9<Entity | null>;
export declare type Entity = {
    type: "しょ";
    side: Side;
    prof: ShogiProfession;
    can_kumal: boolean;
} | {
    type: "碁";
    side: Side;
} | {
    type: "ス";
    side: Side;
    prof: ChessProfession;
    never_moved: boolean;
    subject_to_en_passant?: true;
} | {
    type: "王";
    side: Side;
    prof: KingProfession;
    has_moved_only_once: boolean;
    never_moved: boolean;
};
export declare type Profession = KingProfession | ShogiProfession | ChessProfession;
export declare type UnpromotedShogiProfession = "香" | "桂" | "銀" | "金";
export declare type ShogiProfession = UnpromotedShogiProfession | "成香" | "成桂" | "成銀";
export declare function unpromote(a: ShogiProfession): UnpromotedShogiProfession;
export declare function professionFullName(a: Profession): string;
export declare function isUnpromotedShogiProfession(a: unknown): a is UnpromotedShogiProfession;
export declare type ChessProfession = "ク" | "ル" | "ナ" | "ビ" | "ポ" | "と";
export declare type KingProfession = "キ" | "超";
export declare function is_promotable(prof: Profession): boolean;
export declare type PiecePhaseMove = {
    side: Side;
    from?: Coordinate | "打" | "右" | "左";
    to: Coordinate;
    prof: ShogiProfession | ChessProfession | KingProfession;
    promotes?: boolean;
};
export declare type Move = {
    piece_phase: PiecePhaseMove;
    stone_to?: Coordinate;
};
